package domain

import (
	"testing"

	"github.com/test-go/testify/assert"
	"github.com/wormhole-foundation/wormhole/sdk/vaa"
)

func TestEncodeTrxHashByChainID(t *testing.T) {
	var tests = []struct {
		chainID vaa.ChainID
		txHash  []byte
		want    string
		err     error
	}{
		{
			chainID: vaa.ChainIDSolana,
			txHash:  []byte{0x23, 0xac, 0x49, 0x94, 0x37, 0xa8, 0xe6, 0x53, 0x3b, 0x79, 0x0d, 0x55, 0x78, 0xaf, 0x5d, 0x39, 0xb3, 0x49, 0x88, 0x31, 0x88, 0xec, 0xa5, 0x35, 0xb9, 0x57, 0xd8, 0x2a, 0x0e, 0x77, 0xeb, 0x03},
			want:    "3QFeCHsG9WDXzMozWyck8RUxmw59jyj7MPnQd4w2mbDL",
			err:     nil,
		},
		{
			chainID: vaa.ChainIDAlgorand,
			txHash:  []byte{0xd3, 0x45, 0x59, 0x5e, 0x2a, 0x0f, 0xab, 0x5c, 0xde, 0x71, 0x20, 0xb6, 0xbe, 0xb6, 0xee, 0x0b, 0xb9, 0x4b, 0x57, 0x8a, 0xa5, 0x69, 0x95, 0x2d, 0x00, 0x0c, 0xe8, 0xbf, 0xef, 0x03, 0x2d, 0x22},
			want:    "2NCVSXRKB6VVZXTREC3L5NXOBO4UWV4KUVUZKLIABTUL73YDFURA",
			err:     nil,
		},
		{
			chainID: vaa.ChainIDEthereum,
			txHash:  []byte{0xb9, 0x11, 0xcb, 0xfb, 0x0e, 0x42, 0xc5, 0x04, 0x77, 0x2b, 0xe9, 0x16, 0xbb, 0xeb, 0x8a, 0x46, 0xfc, 0xe7, 0x2b, 0xe5, 0xc6, 0x11, 0x28, 0xe7, 0x12, 0x93, 0x68, 0x26, 0x32, 0x88, 0xcc, 0x7d},
			want:    "b911cbfb0e42c504772be916bbeb8a46fce72be5c61128e7129368263288cc7d",
			err:     nil,
		},
		{
			chainID: vaa.ChainIDNear,
			txHash:  []byte{0x02, 0xde, 0x67, 0xd0, 0x15, 0x34, 0x02, 0x1c, 0x0e, 0x5b, 0x17, 0x68, 0x6e, 0x1e, 0x70, 0xd4, 0x79, 0x39, 0x6d, 0xa2, 0x9d, 0x1e, 0xbc, 0xe4, 0x9a, 0x4c, 0xad, 0xda, 0x4b, 0xca, 0xa3, 0x2b},
			want:    "CCWhFHoDg5eycFJC7EHbYXnNdXW1ed8tjdNHCLbYZEa",
			err:     nil,
		},
		{
			chainID: vaa.ChainIDSui,
			txHash:  []byte{0x29, 0xf4, 0xe6, 0xd8, 0xe0, 0xbf, 0x65, 0x21, 0xe5, 0xf3, 0x30, 0x28, 0x73, 0xa1, 0xf0, 0x08, 0x65, 0xb7, 0xcf, 0xe0, 0x48, 0x36, 0x73, 0x4d, 0x74, 0xed, 0x8c, 0x99, 0x6e, 0x7a, 0x07, 0x86},
			want:    "3pnJrxdjJeDUSvAquDiidApuRLXp5jATdLPyLhjrJsv5",
			err:     nil,
		},
	}

	for _, test := range tests {
		got, err := EncodeTrxHashByChainID(test.chainID, test.txHash)
		assert.Equal(t, test.want, got, "EncodeTrxHashByChainID() = %v, want %v", got, test.want)
		assert.Equal(t, test.err, err, "EncodeTrxHashByChainID() = %v, want %v", err, test.err)
	}
}
